def chunk(array, size=1):
    result = []

    if (size >= len(array)):
        result.append(array)
    else:
        result = append_list(array, size, result)

    return result

def append_list(array, size, result):
    if len(array) == 0:
        return result
    else:
        temp = array[0: size]
        result.append(temp)
        new_array = list(set(array) - set(temp))

        append_list(new_array, size, result)

    return result

def compact(array):
    return [i for i in array if i not in [False, None, '']]

def difference(array, values):
    return list(set(array) - set(values))

def drop(array, number=1):
    return array[number: len(array)]

def dropRight(array, number=1):
    if len(array) < number:
        return []
    return array[0: len(array) - number]

def fill(array, value, start=0, end=None):
    if end is None:
        end = len(array)

    for index, _ in enumerate(array):
        if index >= start and index < end:
            array[index] = value

    return array

def indexOf(array, value, fromIndex=0):
    result = None
    for index, item in enumerate(array):
        if index >= fromIndex and item == value:
            result = index
            break

    return result

def initial(array):
    return [item for index, item in enumerate(array) if index != len(array) - 1]

def pull(array, *args):
    return [i for i in array if i not in args]


__version__ = "0.4.2"

# arrays
from .arrays import (
    chunk,
    compact,
    difference,
    drop,
    dropRight,
    fill,
    indexOf,
    initial,
    pull
)

# maths
from .maths import (
    add,
    ceil,
    divide,
    floor,
    max,
    mean,
    min,
    multiply,
    substract,
    sum
)

# number
from .number import (
    clamp,
    inRange,
    random
)

# string
from .string import (
    camelCase,
    capitalize,
    endsWith,
    escape,
    lowerCase,
    lowerFirst,
    pad,
    padEnd,
    padStart,
    repeat,
    replace,
    startsWith
)


from random import randint, uniform

def clamp(number, lower, upper):
    if check_clamp(number, lower, upper):
        return number
    if check_clamp(lower, number, upper):
        return lower
    if check_clamp(upper, number, lower):
        return upper

def check_clamp(a, b, c):
    return is_between(b, a, c) or is_between(c, a, b)

def is_between(a, b, c):
    return a <= b <= c

def inRange(number, start, end=None):
    if (end == None):
        end = start
        start = 0

    return baseInRange(number, start, end)

def baseInRange(number, start, end):
    return number >= min(start, end) and number < max(start, end)

def random(lower=0, upper=1, floating=False):
    floating = (isinstance(lower, float) or isinstance(upper, float) or floating is True or upper is True)
    if upper < lower:
        upper, lower = lower, upper

    if floating:
        rnd = uniform(lower, upper)
    else:
        rnd = randint(lower, upper)

    return rnd


import re 
import pylodash.arrays as _

def camelCase(string=''):
    items = replace_special_characters(string)

    # format first letter for word
    result = [item.lower() if index == 0 else item[0].upper() + item[1:].lower() for index, item in enumerate(items)]

    return ''.join(result)

def replace_special_characters(string=''):
    # remove all special characters base on pattern
    items = re.split('[-_*)( %^$#@!]', string)

    # remove all elements is empty, None, False
    return _.compact(items)

def capitalize(string=''):
    return string[0].upper() + string[1:].lower()

def endsWith(string='', target=None, position=None):
    if position is None:
        position = len(string)

    return string[position-1] == target

def string_replace(string, old_character, new_character):
    if old_character in string:
        string = string.replace(old_character, new_character)
    return string

def escape(string=''):
    characters = {
        '&': '&amp;',
        '>': '&gt;',
        '"': '&quot;',
        '\'': '&apos;',
        '<': '&lt;'
    }

    for key, value in characters.items():
        string = string_replace(string, key, value)

    return string

def lowerCase(string=''):
    items = replace_special_characters(string)
    result = [i.lower() for i in items]

    return ' '.join(result)

def lowerFirst(string=''):
    return string[0].lower() + string[1:]

def pad_characters(chars, position):
    return (chars*position)[0:position]

def pad(string='', length=0, chars=' '):
    left = length % len(string)
    right = length - len(string) - left
    return pad_characters(chars, left) + string + pad_characters(chars, right)

def padEnd(string='', length=0, chars=' '):
    right = length - len(string)
    return string + pad_characters(chars, right)

def padStart(string='', length=0, chars=' '):
    left = length - len(string)
    return pad_characters(chars, left) + string

def repeat(string='', n=1):
    return string*n

def replace(string, pattern, replacement):
    return string.replace(pattern, replacement)

def startsWith(string, target, position=0):
    return string[position] == target


"""Setup script for pylodash"""

import os.path
from setuptools import setup

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# The text of the README file
with open(os.path.join(HERE, "README.md"), encoding='utf-8') as f:
    README = f.read()

# This call to setup() does all the work
setup(name='pylodash',
    version='0.4.2',
    description='A modern Python utility library delivering modularity, performance & extras.',
    long_description=README,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Text Processing :: Linguistic',
    ],
    keywords='python lodash utilities',
    url='https://gitlab.asoft-python.com/g-tuanluu/python-training',
    author='Tuan Luu',
    author_email='tuan.luu@asnet.com.vn',
    license='MIT',
    packages=['pylodash'],
    install_requires=[
        'markdown',
    ],
    test_suite='nose2.collector.collector',
    tests_require=['nose2'],
    include_package_data=True,
    zip_safe=False)


import math

def add(augend, addend):
    return augend + addend

def ceil(number, precision=0):
    multiplier = 10 ** precision
    return math.ceil(number * multiplier) / multiplier

def divide(dividend, divisor):
    return dividend / divisor

def floor(number, precision=0):
    multiplier = 10 ** precision
    return math.floor(number * multiplier) / multiplier

def max(array):
    if len(array) == 0:
        return None

    max = 0
    for item in array:
        if item > max:
            max = item

    return max

def mean(array):
    return sum(array) / len(array)

def min(array):
    if len(array) == 0:
        return None

    min = array[0]
    for item in array:
        if item < min:
            min = item

    return min

def multiply(multiplier, multiplicand):
    return multiplier * multiplicand

def substract(minuend, subtrahend):
    return minuend - subtrahend

def sum(array):
    result = 0
    for i in array:
        result += i

    return result